@@ -1,5 +1,5 @@
 # coding: utf-8
-from sqlalchemy.testing import eq_, is_, assert_raises, \
+from sqlalchemy.testing import eq_, is_, is_not_, assert_raises, \
     assert_raises_message, expect_warnings
 import decimal
 import datetime
@@ -16,7 +16,7 @@ from sqlalchemy.sql import visitors
 from sqlalchemy import inspection
 from sqlalchemy import exc, types, util, dialects
 from sqlalchemy.util import OrderedDict
-for name in dialects.__all__:
+for name in dialects.__all__:  # noqa
     __import__("sqlalchemy.dialects.%s" % name)
 from sqlalchemy.sql import operators, column, table, null
 from sqlalchemy.schema import CheckConstraint, AddConstraint
@@ -31,8 +31,6 @@ from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import mock
 
 
-
-
 class AdaptTest(fixtures.TestBase):
 
     def _all_dialect_modules(self):
@@ -305,14 +303,14 @@ class UserDefinedTest(fixtures.TablesTest, AssertsCompiledSQL):
             user_id=4, goofy='fred', goofy2='fred', goofy4=util.u('fred'),
             goofy7=util.u('fred'), goofy8=9, goofy9=9)
 
-        l = users.select().order_by(users.c.user_id).execute().fetchall()
+        result = users.select().order_by(users.c.user_id).execute().fetchall()
         for assertstr, assertint, assertint2, row in zip(
             [
                 "BIND_INjackBIND_OUT", "BIND_INlalaBIND_OUT",
                 "BIND_INfredBIND_OUT"],
             [1200, 1500, 900],
             [1800, 2250, 1350],
-            l
+            result
         ):
             for col in list(row)[1:5]:
                 eq_(col, assertstr)
@@ -890,7 +888,6 @@ class TypeCoerceCastTest(fixtures.TablesTest):
             [('BIND_INd1BIND_OUT', )])
 
 
-
 class VariantTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def setup(self):
@@ -1009,6 +1006,55 @@ class VariantTest(fixtures.TestBase, AssertsCompiledSQL):
             'fooUTWO'
         )
 
+    def test_comparator_variant(self):
+        expr = column('x', self.variant) == "bar"
+        is_(
+            expr.right.type, self.variant
+        )
+
+    @testing.only_on("sqlite")
+    @testing.provide_metadata
+    def test_round_trip(self):
+        variant = self.UTypeOne().with_variant(
+            self.UTypeTwo(), 'sqlite')
+
+        t = Table('t', self.metadata,
+                Column('x', variant)
+        )
+        with testing.db.connect() as conn:
+            t.create(conn)
+
+            conn.execute(
+                t.insert(),
+                x='foo'
+            )
+
+            eq_(
+                conn.scalar(select([t.c.x]).where(t.c.x == 'foo')),
+                'fooUTWO'
+            )
+
+    @testing.only_on("sqlite")
+    @testing.provide_metadata
+    def test_round_trip_sqlite_datetime(self):
+        variant = DateTime().with_variant(
+            dialects.sqlite.DATETIME(truncate_microseconds=True), 'sqlite')
+
+        t = Table('t', self.metadata,
+                Column('x', variant)
+        )
+        with testing.db.connect() as conn:
+            t.create(conn)
+
+            conn.execute(
+                t.insert(),
+                x=datetime.datetime(2015, 4, 18, 10, 15, 17, 4839)
+            )
+
+            eq_(
+                conn.scalar(select([t.c.x]).where(t.c.x == datetime.datetime(2015, 4, 18, 10, 15, 17, 1059))),
+                datetime.datetime(2015, 4, 18, 10, 15, 17)
+            )
 
 class UnicodeTest(fixtures.TestBase):
 
@@ -1120,9 +1166,9 @@ class EnumTest(AssertsCompiledSQL, fixtures.TablesTest):
             Column("id", Integer, primary_key=True),
             Column('someenum', Enum('one', 'two', 'three', native_enum=False)),
             Column('someotherenum',
-                Enum('one', 'two', 'three',
-                     create_constraint=False, native_enum=False,
-                     validate_strings=True)),
+                   Enum('one', 'two', 'three',
+                        create_constraint=False, native_enum=False,
+                        validate_strings=True)),
         )
 
         Table(
@@ -1318,19 +1364,79 @@ class EnumTest(AssertsCompiledSQL, fixtures.TablesTest):
                 non_native_enum_table.insert(), {"id": 1, "someenum": None})
             eq_(conn.scalar(select([non_native_enum_table.c.someenum])), None)
 
-
-    @testing.fails_on(
-        'mysql',
-        "The CHECK clause is parsed but ignored by all storage engines.")
-    @testing.fails_on(
-        'mssql', "FIXME: MS-SQL 2005 doesn't honor CHECK ?!?")
+    @testing.requires.enforces_check_constraints
     def test_check_constraint(self):
         assert_raises(
-            (exc.IntegrityError, exc.ProgrammingError),
+            (
+                exc.IntegrityError, exc.ProgrammingError,
+                exc.OperationalError,
+                # PyMySQL raising InternalError until
+                # https://github.com/PyMySQL/PyMySQL/issues/607 is resolved
+                exc.InternalError),
             testing.db.execute,
             "insert into non_native_enum_table "
             "(id, someenum) values(1, 'four')")
 
+    @testing.requires.enforces_check_constraints
+    @testing.provide_metadata
+    def test_variant_we_are_default(self):
+        # test that the "variant" does not create a constraint
+        t = Table(
+            'my_table', self.metadata,
+            Column(
+                'data', Enum("one", "two", "three",
+                             native_enum=False, name="e1").with_variant(
+                    Enum("four", "five", "six", native_enum=False,
+                         name="e2"), "some_other_db"
+                )
+            ),
+            mysql_engine='InnoDB'
+        )
+
+        eq_(
+            len([c for c in t.constraints if isinstance(c, CheckConstraint)]),
+            2
+        )
+
+        with testing.db.connect() as conn:
+            self.metadata.create_all(conn)
+            assert_raises(
+                (exc.DBAPIError, ),
+                conn.execute,
+                "insert into my_table "
+                "(data) values('four')")
+            conn.execute("insert into my_table (data) values ('two')")
+
+    @testing.requires.enforces_check_constraints
+    @testing.provide_metadata
+    def test_variant_we_are_not_default(self):
+        # test that the "variant" does not create a constraint
+        t = Table(
+            'my_table', self.metadata,
+            Column(
+                'data', Enum("one", "two", "three", native_enum=False,
+                             name="e1").with_variant(
+                    Enum("four", "five", "six", native_enum=False, name="e2"),
+                    testing.db.dialect.name
+                )
+            )
+        )
+
+        # ensure Variant isn't exploding the constraints
+        eq_(
+            len([c for c in t.constraints if isinstance(c, CheckConstraint)]),
+            2
+        )
+
+        with testing.db.connect() as conn:
+            self.metadata.create_all(conn)
+            assert_raises(
+                (exc.DBAPIError, ),
+                conn.execute,
+                "insert into my_table "
+                "(data) values('two')")
+            conn.execute("insert into my_table (data) values ('four')")
+
     def test_skip_check_constraint(self):
         with testing.db.connect() as conn:
             conn.execute(
@@ -1466,6 +1572,7 @@ class EnumTest(AssertsCompiledSQL, fixtures.TablesTest):
             "Enum('x', 'y', name='somename', "
             "inherit_schema=True, native_enum=False)")
 
+
 binary_table = MyPickleType = metadata = None
 
 
@@ -1536,14 +1643,14 @@ class BinaryTest(fixtures.TestBase, AssertsExecutionResults):
                     'data': LargeBinary, 'data_slice': LargeBinary},
                 bind=testing.db)
         ):
-            l = stmt.execute().fetchall()
-            eq_(stream1, l[0]['data'])
-            eq_(stream1[0:100], l[0]['data_slice'])
-            eq_(stream2, l[1]['data'])
-            eq_(testobj1, l[0]['pickled'])
-            eq_(testobj2, l[1]['pickled'])
-            eq_(testobj3.moredata, l[0]['mypickle'].moredata)
-            eq_(l[0]['mypickle'].stuff, 'this is the right stuff')
+            result = stmt.execute().fetchall()
+            eq_(stream1, result[0]['data'])
+            eq_(stream1[0:100], result[0]['data_slice'])
+            eq_(stream2, result[1]['data'])
+            eq_(testobj1, result[0]['pickled'])
+            eq_(testobj2, result[1]['pickled'])
+            eq_(testobj3.moredata, result[0]['mypickle'].moredata)
+            eq_(result[0]['mypickle'].stuff, 'this is the right stuff')
 
     @testing.requires.binary_comparisons
     def test_comparison(self):
@@ -1721,6 +1828,7 @@ class JSONTest(fixtures.TestBase):
         bindproc = expr.right.type._cached_literal_processor(non_str_dialect)
         eq_(bindproc(expr.right.value), "'five'")
 
+
 class ArrayTest(fixtures.TestBase):
 
     def _myarray_fixture(self):
@@ -1993,12 +2101,52 @@ class ExpressionTest(
             use_default_dialect=True
         )
 
-        assert expr.type.__class__ is MyTypeDec
+        is_(expr.right.type.__class__, MyTypeDec)
+        is_(expr.type.__class__, MyTypeDec)
+
         eq_(
             testing.db.execute(select([expr.label('foo')])).scalar(),
             "BIND_INfooBIND_IN6BIND_OUT"
         )
 
+    def test_variant_righthand_coercion_honors_wrapped(self):
+        my_json_normal = JSON()
+        my_json_variant = JSON().with_variant(String(), "sqlite")
+
+        tab = table(
+            'test',
+            column('avalue', my_json_normal),
+            column('bvalue', my_json_variant)
+        )
+        expr = tab.c.avalue['foo'] == 'bar'
+
+        is_(expr.right.type._type_affinity, String)
+        is_not_(expr.right.type, my_json_normal)
+
+        expr = tab.c.bvalue['foo'] == 'bar'
+
+        is_(expr.right.type._type_affinity, String)
+        is_not_(expr.right.type, my_json_variant)
+
+    def test_variant_righthand_coercion_returns_self(self):
+        my_datetime_normal = DateTime()
+        my_datetime_variant = DateTime().with_variant(
+            dialects.sqlite.DATETIME(truncate_microseconds=False), "sqlite")
+
+        tab = table(
+            'test',
+            column('avalue', my_datetime_normal),
+            column('bvalue', my_datetime_variant)
+        )
+        expr = tab.c.avalue == datetime.datetime(2015, 10, 14, 15, 17, 18)
+
+        is_(expr.right.type._type_affinity, DateTime)
+        is_(expr.right.type, my_datetime_normal)
+
+        expr = tab.c.bvalue == datetime.datetime(2015, 10, 14, 15, 17, 18)
+
+        is_(expr.right.type, my_datetime_variant)
+
     def test_bind_typing(self):
         from sqlalchemy.sql import column
 
@@ -2016,7 +2164,7 @@ class ExpressionTest(
         # untyped bind - it gets assigned MyFoobarType
         bp = bindparam("foo")
         expr = column("foo", MyFoobarType) + bp
-        assert bp.type._type_affinity is types.NullType
+        assert bp.type._type_affinity is types.NullType  # noqa
         assert expr.right.type._type_affinity is MyFoobarType
 
         expr = column("foo", MyFoobarType) + bindparam("foo", type_=Integer)
@@ -2115,7 +2263,6 @@ class ExpressionTest(
         )
 
 
-
 class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = 'default'
 
@@ -2276,6 +2423,7 @@ class NumericRawSQLTest(fixtures.TestBase):
         else:
             eq_(val, 46.583)
 
+
 interval_table = metadata = None
 
 
@@ -2423,7 +2571,8 @@ class BooleanTest(
         boolean_table = self.tables.boolean_table
         with testing.db.connect() as conn:
             conn.execute(
-                "insert into boolean_table (id, unconstrained_value) values (1, 5)"
+                "insert into boolean_table (id, unconstrained_value) "
+                "values (1, 5)"
             )
 
             eq_(
@@ -2436,6 +2585,7 @@ class BooleanTest(
                 True
             )
 
+
 class PickleTest(fixtures.TestBase):
 
     def test_eq_comparison(self):
@@ -2463,6 +2613,7 @@ class PickleTest(fixtures.TestBase):
         ):
             assert p1.compare_values(p1.copy_value(obj), obj)
 
+
 meta = None
 
 
